/*
 * isa.c
 *
 *  Created on: 02-03-2012
 *      Author: Polprzewodnikowy
 */

#include <stdint.h>
#include "isa.h"
#include "gpio.h"
#include "hdr/hdr_gpio.h"

void ISA_Init(void)
{
	ISA_Set_Data_IO(ISA_OUTPUT);

	gpio_pin_cfg(ADDR0_GPIO, ADDR0_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR1_GPIO, ADDR1_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR2_GPIO, ADDR2_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR3_GPIO, ADDR3_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR4_GPIO, ADDR4_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR5_GPIO, ADDR5_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR6_GPIO, ADDR6_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR7_GPIO, ADDR7_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR8_GPIO, ADDR8_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(ADDR9_GPIO, ADDR9_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);

	gpio_pin_cfg(RESET_GPIO, RESET_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);

	gpio_pin_cfg(ALE_GPIO, ALE_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(AEN_GPIO, AEN_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);

	gpio_pin_cfg(IOR_GPIO, IOR_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(IOW_GPIO, IOW_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);

	CLR_RESET;				//reset state
	CLR_ALE;				//reset state
	CLR_AEN;				//reset state
	SET_IOW;				//reset state
	SET_IOR;				//reset state

	ISA_Write_Data(0);		//reset state
	ISA_Write_Addr(0);		//reset state

	SET_RESET;				//reset card
	CLR_RESET;
	return;
}

void ISA_Set_Data_IO(io_typedef f_io)
{
	if(f_io == ISA_INPUT)
	{
		DATA_GPIO->CRL = 0x44444444;
		DATA_GPIO->CRH = 0x44444444;
	}
	else if(f_io == ISA_OUTPUT)
	{
		DATA_GPIO->CRL = 0x33333333;
		DATA_GPIO->CRH = 0x33333333;
	}
	return;
}

void ISA_Write_Data(uint16_t f_data)
{
	DATA_GPIO->ODR = f_data;
	return;
}

uint16_t ISA_Read_Data(void)
{
	return DATA_GPIO->IDR;
}

void ISA_Write_Addr(uint32_t f_addr)
{
	if(f_addr & (1<<0))	SET_ADDR0;
	else 				CLR_ADDR0;

	if(f_addr & (1<<1))	SET_ADDR1;
	else 				CLR_ADDR1;

	if(f_addr & (1<<2))	SET_ADDR2;
	else 				CLR_ADDR2;

	if(f_addr & (1<<3))	SET_ADDR3;
	else 				CLR_ADDR3;

	if(f_addr & (1<<4))	SET_ADDR4;
	else 				CLR_ADDR4;

	if(f_addr & (1<<5))	SET_ADDR5;
	else 				CLR_ADDR5;

	if(f_addr & (1<<6))	SET_ADDR6;
	else 				CLR_ADDR6;

	if(f_addr & (1<<7))	SET_ADDR7;
	else 				CLR_ADDR7;

	if(f_addr & (1<<8))	SET_ADDR8;
	else 				CLR_ADDR8;

	if(f_addr & (1<<9))	SET_ADDR9;
	else 				CLR_ADDR9;
	return;
}

void ISA_Write_IO(uint32_t f_addr, uint16_t f_data)
{
	ISA_Set_Data_IO(ISA_OUTPUT);
	ISA_Write_Addr(f_addr);
	SET_ALE;
	ISA_Write_Data(f_data);
	CLR_IOW;
	asm("nop");
	SET_IOW;
	CLR_ALE;
	return;
}

uint16_t ISA_Read_IO(uint32_t f_addr)
{
	ISA_Set_Data_IO(ISA_INPUT);
	ISA_Write_Addr(f_addr);
	SET_ALE;
	CLR_IOR;
	asm("nop");
	uint16_t f_data = ISA_Read_Data();
	SET_IOR;
	CLR_ALE;
	return f_data;
}

void ISA_DMA_Write_IO(uint32_t f_addr, uint16_t f_data)
{
	ISA_Set_Data_IO(ISA_OUTPUT);
	ISA_Write_Addr(f_addr);
	SET_AEN;
	ISA_Write_Data(f_data);
	CLR_IOW;
	asm("nop");
	SET_IOW;
	CLR_AEN;
	return;
}

uint16_t ISA_DMA_Read_IO(uint32_t f_addr)
{
	ISA_Set_Data_IO(ISA_INPUT);
	ISA_Write_Addr(f_addr);
	SET_AEN;
	CLR_IOR;
	asm("nop");
	uint16_t f_data = ISA_Read_Data();
	SET_IOR;
	CLR_AEN;
	return f_data;
}

