/*
 * isa_sb16.c
 *
 *  Created on: 20-01-2012
 *      Author: Polprzewodnikowy
 */

#include <stdint.h>
#include "isa.h"
#include "sb16.h"
#include "gpio.h"
#include "hdr/hdr_gpio.h"

void SB_Init(void)
{
	gpio_pin_cfg(IRQ5_GPIO, IRQ5_PIN, GPIO_CRx_MODE_CNF_IN_PULL_U_D_value);

	gpio_pin_cfg(DRQ0_GPIO, DRQ0_PIN, GPIO_CRx_MODE_CNF_IN_PULL_U_D_value);
	gpio_pin_cfg(DRQ5_GPIO, DRQ5_PIN, GPIO_CRx_MODE_CNF_IN_PULL_U_D_value);

	gpio_pin_cfg(DACK0_GPIO, DACK0_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);
	gpio_pin_cfg(DACK5_GPIO, DACK5_PIN, GPIO_CRx_MODE_CNF_OUT_PP_50M_value);

	IRQ5_GPIO->BRR = (1<<IRQ5_PIN);
	AFIO->EXTICR[1] |= (0b0001<<(1*4));
	EXTI->PR |= (1<<IRQ5_PIN);
	EXTI->RTSR |= (1<<IRQ5_PIN);
	EXTI->IMR |= (1<<IRQ5_PIN);
	NVIC_EnableIRQ(EXTI9_5_IRQn);

	DRQ0_GPIO->BRR = (1<<DRQ0_PIN);
	AFIO->EXTICR[0] |= (0b0001<<(1*4));
	EXTI->PR |= (1<<DRQ0_PIN);
	EXTI->RTSR |= (1<<DRQ0_PIN);
	EXTI->IMR |= (1<<DRQ0_PIN);

	DRQ5_GPIO->BRR = (1<<DRQ5_PIN);
	AFIO->EXTICR[0] |= (0b0001<<(2*4));
	EXTI->PR |= (1<<DRQ5_PIN);
	EXTI->RTSR |= (1<<DRQ5_PIN);
	EXTI->IMR |= (1<<DRQ5_PIN);

	SET_DACK0;									//reset state
	SET_DACK5;									//reset state

	SB_Mixer_Write(0x00, 0xFF);					//Mixer reset

	SB_Mixer_Write(0x80, (1<<1));				//IRQ configure, IRQ5
	SB_Mixer_Write(0x81, (1<<5)|(1<<0));		//DMA configure, DMA5 & DMA0

	SB_Mixer_Write(0x30, 0xF8);					//Master L volume, max
	SB_Mixer_Write(0x31, 0xF8);					//Master R volume, max

	SB_DSP_Reset();
	return;
}

void SB_DSP_Reset(void)
{
	ISA_Write_IO(BASE_ADDR|0x6,1);		//Reset sequence
	ISA_Write_IO(BASE_ADDR|0x6,0);
	uint16_t p;
	for(p = 0; p<100; p++)
	{
		if((SB_DSP_Read() & 0xFF) == 0xAA)
			break;
	}
	SB_DSP_Write(0xD1);					//Turn on speaker in DSP
	for(p = 0xFFFF; p>0; --p);
	return;
}

void SB_DSP_SetTimeConst(uint32_t f_sampling)
{
	SB_DSP_Write(0x41);
	SB_DSP_Write((f_sampling>>8) & 0xFF);
	SB_DSP_Write(f_sampling & 0xFF);
	return;
}

void SB_DSP_DMA_Config(uint8_t f_depth, uint8_t f_channels, uint32_t f_block_size)
{
	if(f_depth == 8)
	{
		SB_DSP_Write(0xC6);
		if(f_channels == 1)
			SB_DSP_Write(0x00);
		else if(f_channels == 2)
			SB_DSP_Write(0x20);
		NVIC_EnableIRQ(EXTI1_IRQn);
	}
	else if(f_depth == 16)
	{
		SB_DSP_Write(0xB6);
		if(f_channels == 1)
			SB_DSP_Write(0x10);
		else if(f_channels == 2)
			SB_DSP_Write(0x30);
		NVIC_EnableIRQ(EXTI2_IRQn);
		f_block_size = (f_block_size>>1);
	}
	SB_DSP_Write((f_block_size-1) & 0xFF);
	SB_DSP_Write(((f_block_size-1)>>8) & 0xFF);
	return;
}

void SB_DSP_DMA_Close(uint8_t f_depth)
{
	if(f_depth == 8)
	{
		NVIC_DisableIRQ(EXTI1_IRQn);
		SB_DSP_Write(0xDA);
	}
	else if(f_depth == 16)
	{
		NVIC_DisableIRQ(EXTI2_IRQn);
		SB_DSP_Write(0xD9);
	}
	return;
}

void SB_Mixer_Write(uint8_t f_addr, uint8_t f_data)
{
	ISA_Write_IO(BASE_ADDR|0x4, f_addr);
	ISA_Write_IO(BASE_ADDR|0x5, f_data);
	return;
}

uint8_t SB_Mixer_Read(uint8_t f_addr)
{
	ISA_Write_IO(BASE_ADDR|0x4, f_addr);
	return ISA_Read_IO(BASE_ADDR|0x5);
}

void SB_DSP_Write(uint16_t f_dsp_data)
{
	while((ISA_Read_IO(BASE_ADDR|0xC) & (1<<7)) != 0);
	ISA_Write_IO(BASE_ADDR|0xC, f_dsp_data);
	return;
}

uint16_t SB_DSP_Read(void)
{
	while((ISA_Read_IO(BASE_ADDR|0xE) & (1<<7)) != (1<<7));
	return ISA_Read_IO(BASE_ADDR|0xA);
}
